"use strict";
// Copyright (c) Adam Fekete
// Distributed under the terms of the Modified BSD License.
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsmolView = exports.JsmolModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const version_1 = require("./version");
// Import the CSS
require("../css/widget.css");
const screenfull = __importStar(require("screenfull"));
// Custom Model. Custom widgets models must at least provide default values
// for model attributes, including
//
//  - `_view_name`
//  - `_view_module`
//  - `_view_module_version`
//
//  - `_model_name`
//  - `_model_module`
//  - `_model_module_version`
//
//  when different from the base class.
// When serialiazing the entire widget state for embedding, only values that
// differ from the defaults will be specified.
class JsmolModel extends base_1.DOMWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: JsmolModel.model_name, _model_module: JsmolModel.model_module, _model_module_version: JsmolModel.model_module_version, _view_name: JsmolModel.view_name, _view_module: JsmolModel.view_module, _view_module_version: JsmolModel.view_module_version, _info: {}, _loaded: false });
    }
}
exports.JsmolModel = JsmolModel;
JsmolModel.serializers = Object.assign({}, base_1.DOMWidgetModel.serializers);
JsmolModel.model_name = 'JsmolModel';
JsmolModel.model_module = version_1.MODULE_NAME;
JsmolModel.model_module_version = version_1.MODULE_VERSION;
JsmolModel.view_name = 'JsmolView'; // Set to null if no view
JsmolModel.view_module = version_1.MODULE_NAME; // Set to null if no view
JsmolModel.view_module_version = version_1.MODULE_VERSION;
/**
 * Renders the widget view.
 *
 * Extra methods provided by the superclass:
 * - send(content: {}, buffers?: ArrayBuffer[] | ArrayBufferView[]): void;
 * - touch(): void; ??
 * - remove(): any; ??
 */
class JsmolView extends base_1.DOMWidgetView {
    /**
     * Initializer, called at the end of the constructor.
     */
    initialize(parameters) {
        super.initialize(parameters);
        // Observe changes in the value traitlet in Python, and define a custom callback.
        // this.model.on('change:_info', this.update, this);
        this.model.on('msg:custom', this.on_custom_message, this);
        this.model.on('destroy', this.on_destroy, this);
    }
    /**
     * Triggered on model change.
     *
     * Update view to be consistent with this.model. The model may have been
     * changed by another view or by a state update from the back-end.
     */
    update(options) {
        // console.log("DEBUG: update");
        // console.log(options);
        super.update(options);
    }
    /**
     * Render a view
     *
     * Construct a Jsmol applet for this view. Each view should have an unique jsmol applet to work.
     * @returns the view or a promise to the view.
     */
    // Defines how the widget gets rendered into the DOM
    render() {
        // jsmol_id should be a valid js variable name because it will be used to generate the actual object
        let jsmol_id = "jsmol_" + this.cid;
        // http://wiki.jmol.org/index.php/Jmol_JavaScript_Object/Info
        let info = Object.assign({ width: "100%", height: "100%", use: "HTML5", j2sPath: j2sPath }, this.model.get('_info'));
        // Do not insert new applets automatically
        Jmol.setDocument(false);
        // Create the main Jsmol applet
        this._applet = Jmol.getApplet(jsmol_id, info);
        // Finally the the content of the div should be generated
        this.el.innerHTML = Jmol.getAppletHtml(this._applet);
        // Jmol rely on this script being implicitly executed, but this is not
        // the case when using innerHTML (compared to jquery .html()). So let's
        // manually execute it
        Jmol.coverApplet(this._applet);
        this.model.set('_loaded', true);
        this.touch();
    }
    /**
     * Using custom massage help to implement simple one-way communication
     */
    on_custom_message(msg) {
        // console.log("DEBUG: on_custom_message");
        // console.log(msg);
        if (msg.type == 'call') {
            switch (msg.func) {
                case "fullscreen":
                    this.fullscreen();
                    break;
                case 'script':
                    this.script(msg.data);
                    break;
                case 'evaluate':
                    this.send({
                        'type': 'response',
                        'func': 'evaluate',
                        'data': this.evaluate(msg.data)
                    });
                    break;
                case 'property':
                    this.send({
                        'type': 'response',
                        'func': 'property',
                        'data': this.property(msg.data)
                    });
                    break;
                default:
                    console.log('there is no method for ' + msg.func);
                    break;
            }
        }
    }
    /**
     * Run the given `command` for this viewer.
     */
    script(command) {
        Jmol.script(this._applet, command);
    }
    /**
     * Evaluate the given commands using JSmol and return the corresponding
     * value. This calls `Jmol.evaluateVar` behind the scenes.
     */
    evaluate(command) {
        return Jmol.evaluateVar(this._applet, command);
    }
    /**
     * Returning with th result of `getPropertyAsArray` for a given expression
     */
    property(command) {
        return Jmol.getPropertyAsArray(this._applet, command);
    }
    /**
     * Show this viewer in fullscreen.
     */
    fullscreen() {
        if (screenfull.isEnabled) {
            screenfull.request(this.el);
        }
    }
    /**
     * The main purpose of this callback to remove jsmol applet object from memory when it has been closed.
     * Important: this callback will be only called by the close() or close_all() python functions.
     */
    on_destroy() {
        // console.log("DEBUG: on_destroy");
        // console.log(this);
        // Removing jsmol applet
        delete window[this._applet._id];
        delete this._applet;
    }
}
exports.JsmolView = JsmolView;
//# sourceMappingURL=widget.js.map